function [numClosed,denClosed] = feedback(numG,denG,numK,denK,varargin)
%FEEDBACK Compute the left or right matrix fraction description of the closed loop system.
%
% [NUMCLOSED,DENCLOSED] = FEEDBACK(NUMG,DENG,NUMK,DENK) or 
% [NUMCLOSED,DENCLOSED] = FEEDBACK(NUMG,DENG,NUMK,DENK,'lr') produces closed 
% loop numerator and denominator matrices for the LMF description of standard feedback 
% configuration:
%
%                         +------------+
%            u  ---->o--->|    G(s)    |----+-----  y
%                    ^    +------------+    |
%                  - |                      |
%                    |    +------------+    |
%                    +----|    K(s)    |<---+    
%                         +------------+
% 
% where by default G = DENG^-1 * NUMG and K = NUMK * DENK^-1 and negative feedback
% is considered.
% 
% [NUMCLOSED,DENCLOSED] = FEEDBACK(NUMG,DENG,NUMK,DENK,'rl') produces RMF description
% of the closed loop system for G described by the right matrix fraction DENG^-1 * NUMG, 
% K described by the left matrix fraction NUMK * DENK^-1.
%
% See also: POL/PARALLEL, POL/SERIES

% Author(s):    Z. Hurak   12-18-2000
% Copyright (c) 2000 by PolyX, Ltd.
% $Revision: 1.0.3 $  $Date: 01-24-2000 $

%---------------------------------------------------------------------------------

global PGLOBAL;
eval('PGLOBAL.FORMAT;',...
   'error(''Use PINIT to initialize the Polynomial Toolbox.'');');

%=============================================================================
% Tests for proper inputs: 
%=============================================================================

ni = nargin;

if ni < 4
   error('Not enough input arguments.');
end

if ni > 5
    error('Too many input arguments.');
end

eval('numG=pol(numG);numK=pol(numK);denG=pol(denG);denK=pol(denK);',...
    'error(''The first four input arguments should be polynomial matrices.'')')

frac = 'lr'; % default: G(s) defined as LMF, K(s) as RMF

if ni == 5
    inpA = varargin{1};
    if ischar(inpA) & (strcmp(inpA,'lr') | strcmp(inpA,'rl'))
        frac = inpA;
    else
        error('The fifth input argument can be either ''lr'' or ''rl''');
    end
end

[sizAnumG,sizBnumG] = size(numG);
[sizAdenG,sizBdenG] = size(denG);
[sizAnumK,sizBnumK] = size(numK);
[sizAdenK,sizBdenK] = size(denK);

if (sizAdenG ~= sizBdenG) | issingular(denG)
    error('The demoninator matrix must be square and full rank.');
end
 
if (sizAdenK ~= sizBdenK) | issingular(denK)
    error('The demoninator matrix must be square and full rank.');
end

if ~isequal([sizAnumK,sizBnumK],[sizBnumG,sizAnumG])
   error('Inconsistent matrix dimensions.');
end

% tady jeste musi byt test na LR: D i N stejny pocet sloupcu a RL: D i N stejny pocet radku

%=============================================================================
% Computation of the closed loop numerator and denominator matrices: 
%=============================================================================

switch frac
case 'lr'
    numAuxRight = denK;                       
    denAuxRight = denG*denK + numG*numK;
    [numAuxLeft,denAuxLeft] = rmf2lmf(numAuxRight,denAuxRight);
    numClosed = numAuxLeft*numG;
    denClosed = denAuxLeft;              % returns LMF description
    
case 'rl'
    numAuxRight = denK;
    denAuxRight = denK*denG + numK*numG;
    [numAuxLeft,denAuxLeft] = lmf2rmf(numAuxRight,denAuxRight);
    numClosed = numG*numAuxRight;
    denClosed = denAuxLeft;              % returns RMF description
end

% end of ../feedback.m


    

    


